# 凯普顿 前端工具集合
## CHANGELOG
###v0.0.14
1. 暴露domUtils工具
2. 去除is_js打包到目标js中
3. 修改grid-autosize指令，使其更兼容ie
4. 修改custom-table，无数据时表头乱了的问题
5. 升级vue iview 等框架
6. 修复complex-table 报错的bug

### v0.0.13
1. 升级iview 为2.0.0正式版
2. 修改modal-ext指令问题

### v0.0.11
1. 列布局左右容器改成了flex容器

### v0.0.10
1. 修复modal-ext 指令报错的问题

### v0.0.9
1. 由于vue-echart动态注册地图数据时问题，移除echarts组件
2. ajax 增加了自定义提示配置
3. 增加模态框支持esc 按键配置

## 指令
### v-grid-autosize
  * iview table组件的自动适应父容器大小

### v-modal-ext
  * iview modal的扩展
  * 支持拖动(darg)
  * 支持非模态(:modal="false")
  * 支持禁用esc按键(:esc-close="false")
  * 参考实例：[drag.html](./examples/drag.html)
  * 需要用到实例中的样式
  ```pug
    Modal(title="测试" v-model="visible"  v-modal-ext="" :modal="false" :drag="true" :esc-close="false")

    Modal(title="测试" v-model="visible"  v-modal-ext="{modal:false,drag:true,escClose:false}" )
  ```

## 插件
### 消息通讯插件（event-hub） 用法和vue的事件api相同
  * 发布消息 this.$pub(messageType,args)
  * 订阅消息 this.$sub(messageType,callback)
  * 取消订阅 this.$unsub(messageType,callback)

## 过滤器
### get
  * 取属性的过滤器
  * `{{obj | get('name')}}`代替`{{obj.name}}` 防止obj为空时报错
  * 文档请参考 [https://lodash.com/docs/4.17.4#get](https://lodash.com/docs/4.17.4#get)

### moment
  * 日期格式过滤器 基于moment库实现
  * 用法 `{{new Date() | moment('YYYY-MM-DD')}}` => `2017-01-01`
  * `{{user | get('birthday') | moment(['1992-07-23'],'isBefore')}}` 调用moment isBefore方法 中括号中是isBefore的参数，其他函数同理
  * 更多请参考 [http://momentjs.com/docs/#/parsing/string-format/](http://momentjs.com/docs/#/parsing/string-format/)


## 工具
`import {ajax,modelUtils,commonYUtils} from 'cpt-toolkit'`
### ajax工具
  * 基于axois针对后端接口规范做的封装
  * 会根据后端返回接口进行提示,如果返回*FAILD*标志会进行错误提示，返回其他默认不会提示。如需提示使用*notify*选项
  * 增加了自定义提示消息配置：notifyMessage
  ```javascript
     ajax.post(urls.save,data,{notify:true,notifyMessage:'自定义提示消息'})
    //强制提示
    ajax.post(urls.save,data,{notify:true})
  ```
  * 会处理登录过期跳转,默认会跳转到后端返回的location,如需自定义处理使用*loginTimeoutHandler*选项
  ```javascript
    //调用到端登录界面
    ajax.defaults.loginTimeoutHandler = (location)=>{
      window._vueInstance.$store.dispatch('account/logout');
    }
  ```
  * 其他用法请参考 [https://github.com/mzabriskie/axios](https://github.com/mzabriskie/axios)


### modelUtils
  * `getColumns`:根据列定义生成iviewtable列定义
  * `getDefaults`:根据列定义生成默认model
  * `getRules`:根据列定义生成表单验证规则

  1. key：字段名称
  2. title：字段标题
  3. order: 再列表中显示的顺序
  4. title：字段标题
  5. default：字段默认值
  6. rules：表单验证规范
  7. align: 靠哪边，可选项：left,right,center
  8. sortable：可排序
  9. width：宽度
  10. render：自定义渲染
  ```javascript
    const fields = [
      {
        key: 'name',
        showInColumn: true,
        order: 1,
        title: '名称',
        default:'测试'
        rules: [
          { required: true, message: '请填写地域名称' }//表单验证规则
        ]
      }
    ]
  ```
### commmonUtils
  * `recurseData`：递归处理数据,最小单元为原始类型的数据
  ```javascript
  //将对象中日期的字段进行格式化
  commmonUtils.recurseData(
    {
      name: 'cpt',
      birthday: new Date(1980, 1, 1),
      child: {
        name: 'cpt-child',
        birthday: new Date(2005, 1, 1)
      }
    },(value,key)=>{
      if(is.date(value)){
        return moment(value).format('YYYY-MM-DD');
      }
      return value;
    }
  )
  //-->
  {
      name: 'cpt',
      birthday: '1980-01-01',
      child: {
        name: 'cpt-child',
        birthday: '2005-01-01'
      }
    }
  ```
  * `recurseItem`:递归处理对象，最小单元为json对象
  ```javascript
  // 将所有菜单项的open属性设置为true
  let menus=[
    {
      name:'类型',children:[
        {name:'苹果'},
        {name:'苹果华为'}
      ]
    }
  ]
  commmonUtils.recurseItem(menus,menu=>{
    menu.open=true;
    return menu;
  })
  //-->
  [
    {
      name:'类型',open:true,children:[
        {name:'苹果',open:true},
        {name:'苹果华为',open:true}
      ]
    }
  ]
  ```

## 组件
### 布局
  * column-layout：列布局 左右支持5列,中间自适应
  ```html
    <column-layout>
      <div slot="left1"></div>
      <div></div>
      <div slot="right1"></div>
    </column-layout>
  ```
  * row-layout：行布局 上下支持5行
  ```html
    <row-layout>
      <div slot="top1"></div>
      <div></div>
      <div slot="bottom1"></div>
    </row-layout>
  ```
  * page-layout:页面布局 分上中下，上又分为左右左边一般作为title区域，右边作为工具栏，下边分页栏等
  ```html
    <page-layout>
      <div slot="top-left"></div>
      <div slot="top-right"></div>
      <div></div>
      <div slot="bottom"></div>
    </page-layout>
  ```
### 图片查看器
  * 可拖动，缩放
  * `<image-view src="./a.png"  />`

### 复杂表格
  * 支持复杂表头
  * 支持汇总栏
  * 增加fixHeight 在自适应的基础上调整高度
  * 扩展了column 配置renderSummary 用法和render相同 不同的是renderSummary的数据是summaryData
  ``` html
   <complex-table ref="table",:columns="columns",:data="data" :summaryData="summaryData" :fixHeight="60" />
   <complex-table :columns="columns" :data="data" />`
  ```
